#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include <despotify.h>
#include <stdlib.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "uiupdate.h"

GtkWidget* dialog;

void login_win () {
	gtk_widget_destroy(dialog);
	login_window = create_login_window ();
	gtk_widget_show (login_window);
}

gboolean pulse_login(despot* dsc) {

	gtk_progress_bar_pulse(login_progress);

	if (dsc->login == 1) {
		return TRUE;
	} else {
		gtk_widget_destroy(login_info);
		return FALSE;
	}
}

int login_pressed (GtkButton *button, gpointer user_data){

	(void)button;
	(void)user_data;
	
	despot* dsc;
	dsc = g_slice_new (despot);

	dsc->login = 1;
	login_info = create_login_info ();
	gtk_widget_show (login_info);
	g_timeout_add(100, (GSourceFunc)pulse_login, dsc);

	gtk_widget_hide (login_window);

	const gchar *user, *pass;
	int remember;

	remember=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton1));
	user=gtk_entry_get_text (GTK_ENTRY (username));
	pass=gtk_entry_get_text (GTK_ENTRY (password));

	if (remember == 0) {
	}

	if (!despotify_init()) {

		dialog = gtk_message_dialog_new(
							0, 
							0, 
							GTK_MESSAGE_ERROR, 
							GTK_BUTTONS_CLOSE, 
							"despotify_init() failed."
							);
		g_signal_connect_swapped (GTK_OBJECT(dialog), "response", G_CALLBACK(login_win), GTK_OBJECT(dialog));
		gtk_widget_show(dialog);
		dsc->login = 0;
		
		return 1;
	}
	
	dsc->ds = despotify_init_client(callback, NULL);

	if (!dsc->ds) {

		dialog = gtk_message_dialog_new(
							0, 
							0, 
							GTK_MESSAGE_ERROR, 
							GTK_BUTTONS_CLOSE, 
							"despotify_init_client() failed"
							);
		g_signal_connect_swapped (GTK_OBJECT(dialog), "response", G_CALLBACK(login_win), GTK_OBJECT(dialog));
		gtk_widget_show(dialog);
		dsc->login = 0;
		
		return 1;
	}

	if (!despotify_authenticate(dsc->ds, user, pass)) {
		
		dialog = gtk_message_dialog_new(
							0, 
							0, 
							GTK_MESSAGE_ERROR, 
							GTK_BUTTONS_CLOSE, 
							"Authentication failed: %s", despotify_get_error(dsc->ds)
							);
		g_signal_connect_swapped (GTK_OBJECT(dialog), "response", G_CALLBACK(login_win), GTK_OBJECT(dialog));
		gtk_widget_show(dialog);
		
		despotify_exit(dsc->ds);
		dsc->login = 0;
		
		return 1;
	}

	print_info(dsc->ds);
	
	GTK_Despotify = create_GTK_Despotify ();
	create_playlists_columns (dsc);
	gtk_widget_show (GTK_Despotify);

	dsc->login = 0;

	/* set window icon */
	GdkPixbuf *icon0;
	icon0 = create_pixbuf ("gtk_despotify_icon_24.png");
	gtk_window_set_icon (GTK_Despotify, icon0);
	
	/* show tray icon */
	ticon(dsc);
	
	g_signal_connect (GTK_Despotify, "delete_event", G_CALLBACK(toggle_window_quit), dsc);
	g_signal_connect (cds_prev, "clicked",G_CALLBACK (prev_track), dsc);
	g_signal_connect (cds_play, "clicked",G_CALLBACK (play_track_button), dsc);
	g_signal_connect (cds_pause, "clicked",G_CALLBACK (pause_track), dsc);
	g_signal_connect (cds_stop, "clicked",G_CALLBACK (stop_track_callback), dsc);
	g_signal_connect (cds_next, "clicked",G_CALLBACK (next_track), dsc);
  	g_signal_connect (button1, "pressed",G_CALLBACK (search_callback), dsc);
	g_signal_connect (search, "activate",G_CALLBACK (search_callback), dsc);
	g_signal_connect (tracktree, "row-activated",G_CALLBACK(play_track_activated), dsc);
	
	return 0;
}

void playlist_show(despot* dsc, struct playlist* p)
{
	
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkListStore  *store;
	struct track *t;
	int i;
	char lenght [60];

	struct track* head = p->tracks;
	dsc->lastlist = p;

	store = gtk_list_store_new (NUMT_COLS, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_FLOAT, G_TYPE_STRING, G_TYPE_STRING);
	
	if (p) {
		if (!head) {
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, COLT_NUMBER, NULL, COLT_TITLE, NULL, COLT_ALBUM, NULL, COLT_LENGHT, NULL, COLT_YEAR, NULL, COLT_POPULARITY, 0.00, COLT_NAME, NULL, COLT_PLAY, NULL, -1);
		}

		int count = 1;
		for (struct track* t = head; t; t = t->next) {
			if (t->has_meta_data) {

				gtk_list_store_append (store, &iter);
				sprintf(lenght, "%2d:%02d", t->length / 60000, t->length % 60000 / 1000);
				gtk_list_store_set (store, &iter,COLT_NUMBER, count, COLT_TITLE, t->title, COLT_ALBUM, t->album, COLT_LENGHT, lenght, COLT_YEAR, t->year, COLT_POPULARITY, t->popularity * 100, -1);

				dsc->lasttrackno = count;
				count++;

				for (struct artist* a = t->artist; a; a = a->next) {
					gtk_list_store_set (store, &iter, COLT_NAME, a->name, COLT_PLAY, t->playable ? "" : "(Unplayable)", -1);
				}

			} else {
				gtk_list_store_append (store, &iter);
				gtk_list_store_set (store, &iter, COLT_NUMBER, NULL, COLT_TITLE, NULL, COLT_ALBUM, NULL, COLT_LENGHT, NULL, COLT_YEAR, NULL, COLT_POPULARITY, 0.00, COLT_NAME, NULL, COLT_PLAY, NULL, -1);
			}
		}
	}

	gtk_tree_view_set_model (GTK_TREE_VIEW (tracktree), GTK_TREE_MODEL (store));
	g_object_unref (store);

}

int update_tracklist_rows(GtkWidget *widget, despot* dsc) {

	(void)widget;

	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkListStore  *store;
	GtkTreeModel *trackmodel;
	struct playlist* rootlist = NULL;
	rootlist = despotify_get_stored_playlists(dsc->ds);
	char lenght [60];
	guint number;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(listtree));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, COLP_NUMBER, &number, -1);
	} else {
		printf("no row selected.\n");
		return 1;
	}

	struct playlist* p = get_playlist(rootlist, number);

	despotify_stop(dsc->ds);
	dsc->stopped=1;

	playlist_show(dsc,p);

	return 0;
}

int play_track_activated(GtkWidget *widget, GtkTreePath *path, GtkTreeViewColumn *column, despot* dsc) {

	(void)widget;
	(void)path;
	(void)column;

	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint number;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tracktree));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, COLT_NUMBER, &number, -1);
	} else {
		printf("no row selected.\n");
		return 1;
	}

	play_track(dsc,number);
	return 0;

}

int play_track_button(GtkWidget *widget, despot* dsc) {

	(void)widget;

	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	guint number;

	if(dsc->ds->snd_session) {
		if(dsc->ds->snd_session->actx->is_paused) {
			despotify_resume(dsc->ds);
			return 0;
		}
	}

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tracktree));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, COLT_NUMBER, &number, -1);
	} else {
		printf("no row selected.\n");
		return 1;
	}

	play_track(dsc,number);

	return 0;

}

int play_track(despot* dsc, guint number) {

	gchar *msg;
	struct track* t = dsc->lastlist->tracks;
	
	for (int i=1; i<number && t; i++) {
	        t = t->next;
	}
	
	if (t) {
		dsc->current_track = t;
		dsc->trackno = number;
		show_current_track_cover(dsc);
		if(despotify_play(dsc->ds, t, FALSE)) {
			dsc->stopped=0;
			if (dsc->looping != 1) {
				dsc->looping=1;
				g_timeout_add(1000, (GSourceFunc)update_progress, dsc);
			}
			return 0;
		} else {
			msg = g_strdup_printf ("Failed to play track: %u", number);
			gtk_statusbar_push (statusbar, 0, msg);
			g_free (msg);
			return 1;
		}
	}

	return 0;

}


int pause_track(GtkWidget *widget, despot* dsc) {

	(void)widget;

	if(dsc->ds->snd_session) {
		if(!dsc->ds->snd_session->actx->is_paused) {
			despotify_pause(dsc->ds);
		} else { 
			despotify_resume(dsc->ds);
		}
	}
}

int stop_track(despot* dsc)
{
	GdkPixbuf *icon0;
	icon0 = create_pixbuf ("sbcov.png");
	gtk_image_set_from_pixbuf(cover, icon0);

	if (despotify_stop(dsc->ds)){
		dsc->stopped=1;
		return 0;
	} else {
		printf("Stop Track Failed!\n");
		return 1;
	}
}

int stop_track_callback(GtkWidget *widget, despot* dsc) {

	(void)widget;

	if (stop_track(dsc)){
		return 0;
	} else {
		return 1;
	}
	
}

int prev_track(GtkWidget *widget, despot* dsc) {

	(void)widget;
	dsc->stopped=1;
	dsc->prevtrack = dsc->trackno;
	
	if (dsc->trackno && dsc->trackno != 1) {
		int i = dsc->trackno - 1;
		play_track(dsc, i);
		return 0;
	} else {
		printf("no previous track\n");
		return 1;
	}
}

int next_track(GtkWidget *widget, despot* dsc) {

	(void)widget;
	dsc->stopped=1;
	dsc->prevtrack = dsc->trackno;

	if (dsc->trackno && dsc->trackno != dsc->lasttrackno) {
		int i = dsc->trackno + 1;
		play_track(dsc, i);
		return 0;
	} else {
		printf("no next track\n");
		int i = 1;
		play_track(dsc, i);
		return 1;
	}
}

void search_callback (GtkWidget *widget, despot* dsc) {

	(void)widget;

	if(strlen(gtk_entry_get_text(GTK_ENTRY(search))) > 0)
	{
		
		despotify_stop(dsc->ds);
		dsc->stopped=1;

		char* searchstring;
				
		searchstring = malloc(sizeof(char) * strlen(gtk_entry_get_text(GTK_ENTRY(search))));
		searchstring = g_strndup(gtk_entry_get_text( GTK_ENTRY(search) ), strlen(gtk_entry_get_text(GTK_ENTRY(search) ) ));
		
		dsc->search = despotify_search(dsc->ds, searchstring, MAX_SEARCH_RESULTS);

		if (!dsc->search) {
			printf("Search failed: %s\n", despotify_get_error(dsc->ds));
		} else {
			dsc->searchlist = dsc->search->playlist;
			playlist_show(dsc, dsc->search->playlist);
		}
	}
}

