#include <gtk/gtk.h>
#include <despotify.h>
#include <stdlib.h>
#include "main.h"
#include "support.h"
#include "uiupdate.h"
#include "interface.h"
#include "callbacks.h"

static GtkWidget *menuitem_prev = NULL;
static GtkWidget *menuitem_pause = NULL;
static GtkWidget *menuitem_stop = NULL;
static GtkWidget *menuitem_next = NULL;
static GtkWidget *menuitem_quit = NULL;

void create_track_columns (despot* dsc) {

	GtkListStore *store;
	GtkTreeIter iter;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *col;

	/* --- Column #0 --- */
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Track", renderer, "text", COLT_NUMBER, NULL);

	/* --- Column #1 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Title", renderer, "text", COLT_TITLE, NULL);

	/* --- Column #2 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Album", renderer, "text", COLT_ALBUM, NULL);

	/* --- Column #3 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Duration", renderer, "text", COLT_LENGHT, NULL);

	/* --- Column #4 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Year", renderer, "text", COLT_YEAR, NULL);

	/* --- Column #5 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Popularity %", renderer, "text", COLT_POPULARITY, NULL);

	/* --- Column #6 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Artist/Band", renderer, "text", COLT_NAME, NULL);

	/* --- Column #7 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tracktree), -1, "Playable", renderer, "text", COLT_PLAY, NULL);


	store = gtk_list_store_new (NUMT_COLS, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT ,G_TYPE_FLOAT, G_TYPE_STRING, G_TYPE_STRING);

	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, COLT_NUMBER, NULL, COLT_TITLE, NULL, COLT_ALBUM, NULL, COLT_LENGHT, NULL, COLT_YEAR, NULL, COLT_POPULARITY, 0.00, COLT_NAME, NULL, COLT_PLAY, NULL, -1);
	gtk_tree_view_set_model (GTK_TREE_VIEW (tracktree), GTK_TREE_MODEL (store));
	g_object_unref (store);

}

void create_playlists_columns (despot* dsc) {

	GtkListStore *store;
	GtkTreeIter iter;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *col;
	dsc->rootlist = despotify_get_stored_playlists(dsc->ds);

	store = gtk_list_store_new (NUMP_COLS, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_STRING);

	/* --- Column #0 --- */
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (listtree), -1, "Number", renderer, "text", COLP_NUMBER, NULL);

	/* --- Column #1 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (listtree), -1, "Name", renderer, "text", COLP_NAME, NULL);

	/* --- Column #2 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (listtree), -1, "Tracks", renderer, "text", COLP_TRACKS, NULL);

	/* --- Column #3 --- */
	col = gtk_tree_view_column_new();
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (listtree), -1, "Author", renderer, "text", COLP_AUTHOR, NULL);


	if (!dsc->rootlist) {
		printf(" <no stored playlists>\n");
	} else {
		int count=1;
		for (struct playlist* p = dsc->rootlist; p; p = p->next) {
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, COLP_NUMBER, count, COLP_NAME, p->name, COLP_TRACKS, p->num_tracks, COLP_AUTHOR, p->author, -1);
			count++;
		}
	}

	gtk_tree_view_set_model (GTK_TREE_VIEW (listtree), GTK_TREE_MODEL (store));
	g_object_unref (store);

	create_track_columns(dsc);

	g_signal_connect(listtree, "cursor-changed", G_CALLBACK(update_tracklist_rows), dsc);

}

gboolean update_progress(despot* dsc) {

	char tooltip [256];
	int ovtime=(int) ov_time_tell (dsc->ds->snd_session->vf);
	int lenght=dsc->current_track->length / 1000;

	if (ovtime >= 0 && dsc->stopped != 1 ) {
		gdouble prog = ((double) ovtime)/lenght;
		gtk_progress_bar_set_fraction (progress,prog);

		gchar *msg;
		msg = g_strdup_printf ("Playing Track No. %i: %s --- %s: (%d:%02d / %d:%02d)", 
					dsc->trackno,
					dsc->current_track->album,
					dsc->current_track->title,
					(ovtime * 1000) / 60000,
					(ovtime * 1000) % 60000 / 1000,
					dsc->current_track->length / 60000, 
					dsc->current_track->length % 60000 / 1000);

		gtk_progress_bar_set_text (progress,msg);
		gtk_statusbar_push (statusbar, 0, msg);
		g_free (msg);

		sprintf(tooltip, "%s\n%s\n(%d:%02d / %d:%02d)",
				dsc->current_track->album,
				dsc->current_track->title,
				(ovtime * 1000) / 60000,
				(ovtime * 1000) % 60000 / 1000,
				dsc->current_track->length / 60000, 
				dsc->current_track->length % 60000 / 1000);

		gtk_status_icon_set_tooltip(tray_icon, tooltip);

	}

	if (dsc->stopped == 1) {
		const gchar *msg;
		msg = g_strdup_printf ("Stopped");
		gtk_progress_bar_set_text (progress,msg);
		gtk_statusbar_push (statusbar, 0, msg);
		g_free (msg);

		gtk_progress_bar_set_fraction (progress,0.00);
		gtk_status_icon_set_tooltip(tray_icon, "stopped");
	}

	if(dsc->ds->snd_session->actx->is_paused) {
		gchar *msg;
		msg = g_strdup_printf ("Paused Track No. %i: %s --- %s: (%d:%02d / %d:%02d)", 
					dsc->trackno,
					dsc->current_track->album,
					dsc->current_track->title,
					(ovtime * 1000) / 60000,
					(ovtime * 1000) % 60000 / 1000,
					dsc->current_track->length / 60000, 
					dsc->current_track->length % 60000 / 1000);

		gtk_progress_bar_set_text (progress,msg);
		gtk_statusbar_push (statusbar, 0, msg);
		g_free (msg);
	}

	if (dsc->stopped == 0 && dsc->ds->snd_session->actx->is_playing == 0 && dsc->lasttime == ovtime) {
		printf("end of track\n");
		dsc->prevtrack = dsc->trackno;

		if (dsc->trackno && dsc->trackno != dsc->lasttrackno) {
			int i = dsc->trackno + 1;
			play_track(dsc, i);
			return TRUE;
		} else {
			printf("no next track\n");
			int i = 1;
			play_track(dsc, i);
			return TRUE;
		}

	}

	dsc->lasttime=ovtime;
	return TRUE;
}


void show_current_track_cover(despot* dsc) {

	static unsigned char* last_track_id;
	struct album_browse* a;
	GdkPixbuf *icon0;
	struct track* t = dsc->current_track;
	last_track_id = t->track_id;
	GdkPixbufLoader *pictureloader;
	pictureloader = gdk_pixbuf_loader_new();
	GError *error = 0;

	if(!last_track_id) {
		last_track_id = malloc(sizeof(char[33]));
	}

	if (t && t->cover_id[0]) {
		int len;
		void* portrait = despotify_get_image(dsc->ds, t->cover_id, &len);
		if (portrait && len) {
			if (pictureloader) {
		        	gdk_pixbuf_loader_write(pictureloader, portrait, len, &error);
				if(!error) {
					GdkPixbuf* pixbuf = gdk_pixbuf_loader_get_pixbuf(pictureloader);
					gtk_image_set_from_pixbuf(cover, pixbuf);
				} else {
					printf("Error loading image: %s\n", error->message);
				}
		    	}
			free(portrait);
		}
	} else {
		printf("Track %s has no portrait.\n", t->title);
		icon0 = create_pixbuf ("sbcov.png");
		gtk_image_set_from_pixbuf(cover, icon0);
	}

}

GtkStatusIcon *create_tray_icon() {

	char tooltip [256];
	GdkPixbuf *icon0;
	tray_icon = gtk_status_icon_new();

	sprintf(tooltip, "GTK Despotify");
	gtk_status_icon_set_tooltip(tray_icon, tooltip);

	icon0 = create_pixbuf ("gtk_despotify_icon_24.png");
	gtk_status_icon_set_from_pixbuf(tray_icon, icon0);

	gtk_status_icon_set_visible(tray_icon, TRUE);
	return tray_icon;

}

static void popup_callback(GObject *widget, guint button, guint activate_time, gpointer user_data)
{

	GtkWidget *menu = user_data;

	gtk_widget_set_sensitive(menuitem_prev,TRUE);
	gtk_widget_set_sensitive(menuitem_pause,TRUE);
	gtk_widget_set_sensitive(menuitem_stop,TRUE);
	gtk_widget_set_sensitive(menuitem_next,TRUE);
	gtk_widget_set_sensitive(menuitem_quit,TRUE);

	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, gtk_status_icon_position_menu, GTK_STATUS_ICON(widget), button, activate_time);

}

static GtkWidget *create_popupmenu(despot* dsc)
{
	GtkWidget *menu;
	GtkWidget *item;
	GtkWidget *image;

	menu = gtk_menu_new();

	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PREVIOUS, GTK_ICON_SIZE_MENU);
	item = gtk_image_menu_item_new_with_label("Prev");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	menuitem_prev = item;
	g_signal_connect(item, "activate",G_CALLBACK(prev_track), dsc);

	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_MENU);
	item = gtk_image_menu_item_new_with_label("Pause");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	menuitem_pause = item;
	g_signal_connect(item, "activate",G_CALLBACK(pause_track), dsc);

	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_STOP, GTK_ICON_SIZE_MENU);
	item = gtk_image_menu_item_new_with_label("Stop");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	menuitem_stop = item;
	g_signal_connect(item, "activate",G_CALLBACK(stop_track_callback), dsc);

	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_NEXT, GTK_ICON_SIZE_MENU);
	item = gtk_image_menu_item_new_with_label("Next");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	menuitem_next = item;
	g_signal_connect(item, "activate",G_CALLBACK(next_track), dsc);

	item = gtk_separator_menu_item_new();
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

	image = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU);
	item = gtk_image_menu_item_new_with_label("Quit");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
	gtk_widget_show(item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	menuitem_quit = item;
	g_signal_connect(item, "activate",G_CALLBACK(quit_ds), dsc);

	return menu;

}

void toggle_window (GtkStatusIcon *status_icon, gpointer user_data) {

	(void)status_icon;

	if (!GTK_WIDGET_VISIBLE(GTK_Despotify)) {
  		gtk_widget_show(GTK_Despotify);
  	} else {
    		gtk_widget_hide (GTK_Despotify);
	}
}

void toggle_window_quit (GtkWidget *widget, GdkEvent *event, despot* dsc) {

	(void)widget;
	(void)event;

	if (!GTK_WIDGET_VISIBLE(GTK_Despotify)) {
  		gtk_widget_show(GTK_Despotify);
  	} else {
    		gtk_widget_hide (GTK_Despotify);
	}
}

void ticon (despot* dsc) {
	GtkWidget *menu;
	tray_icon = create_tray_icon();        
	menu = create_popupmenu(dsc);
	g_signal_connect(G_OBJECT(tray_icon), "popup-menu",G_CALLBACK(popup_callback), menu);
	g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(toggle_window), NULL);
}


void quit_ds (GtkWidget *widget, despot* dsc) {

	(void)widget;
	despotify_exit(dsc->ds);

	if (!despotify_cleanup()) {
		printf("despotify_cleanup() failed\n");
		exit(1);
	}

	gdk_threads_leave();
	gtk_main_quit();

	exit(0);
}





